/*
 * fault.c - Defines the available fault handlers. One example,
 *           fault_random, is provided; you must implement fault_fifo
 *           and fault_lru_clock as well as a 4th handler of your own
 *           choice. Be sure to add your handlers to fault_handlers.
 *
 */

#include <config.h>

#include <vmtrace.h>
#include <fault.h>
#include <options.h>
#include <physmem.h>
#include <stdlib.h>
#include <stdio.h>

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

static void fault_random(pte_t *pte, ref_kind_t type);
static void fault_init_random();

fault_handler_info_t fault_handlers[2] = {
  { "random", fault_random },
  { NULL, NULL } /* last entry must always be NULL/NULL */
};

/* Initialize any state needed by the fault handlers here.
 * For example, the random number generator must be initialized for
 * fault_random. */
void fault_init() {
  fault_init_random();
}

void fault_init_random() {
  long seed = 0;
#ifdef HAVE_GETTIMEOFDAY  
  struct timeval time;
  if (gettimeofday(&time, NULL) == -1) {
    perror("vmtrace: failed to get time to init random number generator");
    abort();
  }
  seed = time.tv_usec;
#else
  fprintf(stderr, "vmtrace: no gettimeofday available, random won't be\n");
#endif
  srandom(seed);
}

void fault_random(pte_t *pte, ref_kind_t type) {
  int page;
  page = random() % opts.phys_pages;
  physmem_evict(page, type);
  physmem_load(page, pte, type);
}


